@@ -313,6 +313,9 @@ public:
 		CHILD_BUSY,
 		CHILD_FAX,
 		CHILD_DND,
+		CHILD_AWAY,
+		CHILD_NOCODEC,
+		CHILD_OFFLINE,
 
 		START_SCRIPT = START_INCOMING,
 		START_SELECTED = START_OUTGOING,
@@ -407,6 +410,9 @@ public:
 		DIAL_FAX,
 		DIAL_PAM,
 		DIAL_DND,
+		DIAL_AWAY,
+		DIAL_OFFLINE,
+		DIAL_NOCODEC,
 
 		DIAL_MACHINE = DIAL_PAM,
 
@@ -1400,6 +1406,12 @@ protected:
 	timeout_t reset_timer, release_timer, flash_timer, interdigit_timer;
 	unsigned answer_count;
 
+	/**
+	 * Virtual to notify driver that a server image reload is
+	 * in progress.
+	 */
+	virtual void reloadDriver(void);
+
 	/**
 	 * Virtual to override method for activating the driver and
 	 * creating all session and span objects associated with it.
@@ -1544,6 +1556,11 @@ public:
 	 */
 	static void stop(void);
 
+	/**
+	 * Notify all drivers about reload.
+	 */
+	static void reload(void);
+
 	/**
 	 * Add session to driver idle list for getIdle, usually during
 	 * stateIdle.
@@ -2283,6 +2300,7 @@ protected:
 	char var_tid[14];
 	char var_sid[16];
 	char var_pid[16];
+	char var_joined[16];
 	char var_rings[4];
 	char var_timeslot[8];
 	char var_spanid[8];
