@@ -903,7 +903,7 @@ bool BayonneSession::stateConnect(Event *event)
 {
 	Event cancel;
 	BayonneSession *s;
-	bool dtmf;
+	bool dtmf, rtn;
 
         if(enterTone(event))
                 return true;
@@ -921,6 +921,12 @@ failed:
                         error("dial-failed");
                 setRunning();
                 return true;
+	case CHILD_OFFLINE:
+		if(!scriptEvent("dial:offline"))
+			if(!scriptEvent("dial:failed"))
+				error("dial-offline");
+		setRunning();
+		return true;
 	case CHILD_INVALID:
 		if(!scriptEvent("dial:invalid"))
 			error("dial-invalid");
@@ -941,10 +947,25 @@ failed:
         case CHILD_BUSY:
                 if(!scriptEvent("dial:busy"))
                         error("dial-busy");
+		setRunning();
                 return true;
+	case CHILD_NOCODEC:
+		if(!scriptEvent("dial:nocodec"))
+			if(!scriptEvent("dial:failed"))
+				error("dial-nocodec");
+		setRunning();
+		return true;
+	case CHILD_AWAY:
+		if(!scriptEvent("dial:away"))
+			if(!scriptEvent("dial:noanswer"))
+				error("dial-away");
+		setRunning();
+		return true;	
 	case CHILD_DND:
 		if(!scriptEvent("dial:dnd"))
-			error("dial-dnd");
+			if(!scriptEvent("dial:busy"))
+				error("dial-dnd");
+		setRunning();
 		return true;
 	case CHILD_FAX:
 		if(!scriptEvent("dial:fax"))
@@ -972,8 +993,17 @@ failed:
                 s = getSid(state.tone.sessionid);
                 if(s)
                         s->queEvent(&cancel);
-	default:
 		return enterCommon(event);
+	default:
+		s = getSid(state.tone.sessionid);
+		rtn = enterCommon(event);
+		if(state.handler != &BayonneSession::stateConnect)
+		{
+			memset(&cancel, 0, sizeof(cancel));
+			cancel.id = STOP_DISCONNECT;
+			s->queEvent(&cancel);
+		}
+		return rtn;
         }
 }
 
@@ -1605,7 +1635,7 @@ seize:
 
 bool BayonneSession::statePickup(Event *event)
 {
-	if(event->id == ENTER_STATE)
+	if(event->id == ENTER_STATE && !offhook)
 		incIncomingAttempts();
 
 	if(enterPickup(event))
@@ -1752,7 +1782,7 @@ bool BayonneSession::stateSeize(Event *event)
 {
 	char evtname[96];
 
-	if(event->id == ENTER_STATE)
+	if(event->id == ENTER_STATE && !offhook)
 		incOutgoingAttempts();
 
 	if(enterSeize(event))
@@ -1822,7 +1852,7 @@ bool BayonneSession::stateSeize(Event *event)
 		return true;
 	case DIAL_DND:
 		seizure = CHILD_DND;
-                if(scriptEvent("call:dnd"))
+                if(scriptEvent("call:dnd") || scriptEvent("call:busy"))
                 {
                         setRunning();
                         return true;
@@ -1842,6 +1872,15 @@ bool BayonneSession::stateSeize(Event *event)
 		seizure = CHILD_RUNNING;
 		setConnecting();
 		return true;
+	case DIAL_AWAY:
+		seizure = CHILD_AWAY;
+		if(scriptEvent("call:away") || scriptEvent("call:noanswer"))
+		{
+			setRunning();
+			return true;
+		}
+		setState(STATE_HANGUP);
+		return true;
 	case DIAL_TIMEOUT:
 noanswer:
 		seizure = CHILD_EXPIRED;
@@ -1855,10 +1894,18 @@ noanswer:
 	case TIMER_EXPIRED:
 		if(state.join.dial)
 			goto noanswer;
+
+	case DIAL_NOCODEC:
+		seizure = CHILD_NOCODEC;
+		goto failure;
+	case DIAL_OFFLINE:
+		seizure = CHILD_OFFLINE;
+		goto failure;
 	case CALL_FAILURE:
 	case DIAL_FAILED:
 failed:
 		seizure = CHILD_FAILED;
+failure:
 		if(scriptEvent("call:failed"))
 		{
 			setRunning();
