@@ -41,7 +41,7 @@ static Keydata::Define driver[] = {
 	{"dtmf", "101"},
 	{"inband", "false"},
 	{"silence", "500"},
-	{"duration", "10m"},
+	{"duration", "3m"},
 	{"invite", "60000"},
 	{"pickup", "2000"},
 	{"hangup", "250"},
@@ -78,9 +78,17 @@ static char *remove_quotes(char *c)
 }
 
 SIPThread *SIPThread::first = NULL;
+SIPTraffic *SIPTraffic::first = NULL;
 Driver Driver::sip;
 unsigned Driver::instance = 0;
 
+const char *Image::dupString(const char *str)
+{
+	char *np = (char *)alloc(strlen(str) + 1);
+	strcpy(np, str);
+	return (const char *)np;
+}
+
 bool Registry::isActive(void)
 {
 	if(!stricmp(type, "anon"))
@@ -89,10 +97,10 @@ bool Registry::isActive(void)
 	if(!proxy)
 		return false;
 
-	if(!active)
+	if(!traffic->active)
 		return false;
 
-	if(getTimer() > 0)
+	if(traffic->getTimer() > 0)
 		return true;
 
 	return false;
@@ -119,7 +127,8 @@ void SIPThread::run(void)
 
 Driver::Driver() :
 BayonneDriver(driver, "/bayonne/driver/sip", "sip", false),
-Thread(atoi(getLast("priority")), atoi(getLast("stack")) * 1024 + 8192)
+Thread(atoi(getLast("priority")), atoi(getLast("stack")) * 1024 + 8192),
+Assoc()
 {
 	const char *cp, *p;
 	char buf[1024];
@@ -271,6 +280,40 @@ Thread(atoi(getLast("priority")), atoi(getLast("stack")) * 1024 + 8192)
 		server->setValue("sip.conf", CONFIG_FILES "/sip.conf");
 }
 
+void *Driver::getMemory(size_t size)
+{
+	return alloc(size);
+}
+
+SIPTraffic *Driver::getTraffic(const char *id)
+{
+	SIPTraffic *traffic = (SIPTraffic *)getPointer(id);
+	if(!traffic)
+	{
+		traffic = (SIPTraffic *)getMemory(sizeof(SIPTraffic));
+		memset(traffic, 0, sizeof(SIPTraffic));
+		setPointer(id, traffic);
+		traffic->next = traffic->first;
+		traffic->first = traffic;
+	}
+	++traffic->refcount;
+	return traffic;
+}
+
+void Driver::reloadDriver(void)
+{
+	SIPTraffic *traffic = SIPTraffic::first;
+
+	while(traffic)
+	{
+		if(traffic->refcount)
+			--traffic->refcount;
+		else 
+			memset(traffic, 0, sizeof(SIPTraffic));
+		traffic = traffic->next;
+	}
+}
+
 bool Driver::isReachable(const char *proxy)
 {
 	char buf[128];
@@ -299,7 +342,7 @@ bool Driver::isReachable(const char *proxy)
 	reg = (Registry *)img->getPointer(buf);
 	if(reg && reg->isActive())
 	{
-		if(!reg->call_limit || reg->active_calls < reg->call_limit)
+		if(!reg->call_limit || reg->traffic->active_calls < reg->call_limit)
 			rtn = true;
 	}
 
@@ -425,8 +468,17 @@ bool Driver::isAvailable(const char *ext)
 	{
 		if(!reg->call_limit)
 			rtn = true;
-		else if(reg->call_limit > reg->active_calls)
+		else if(reg->call_limit > reg->traffic->active_calls)
 			rtn = true;
+		switch(reg->traffic->presence)
+		{
+		case SIPTraffic::P_BUSY:
+		case SIPTraffic::P_AWAY:
+		case SIPTraffic::P_DND:
+			rtn = false;
+		default:
+			break;
+		}
 	}
 	endImage(img);
 	return rtn;
@@ -444,7 +496,7 @@ unsigned Driver::getRegistration(regauth_t *data, unsigned count)
 	{
 		if(!stricmp(reg->type, "anon"))
 			data[idx].remote = "*";
-		else if(!strnicmp(reg->type, "ext", 3) && !reg->active)
+		else if(!strnicmp(reg->type, "ext", 3) && !reg->traffic->active)
 			data[idx].remote = "-";
 		else if(!strnicmp(reg->proxy, "sip:", 4))
 			data[idx].remote = reg->proxy + 4;
@@ -454,22 +506,47 @@ unsigned Driver::getRegistration(regauth_t *data, unsigned count)
 		data[idx].type = reg->type;
 		if(!stricmp(reg->type, "anon"))
 			data[idx].status = "active";
-		else if(!strnicmp(reg->type, "ext", 3) && !reg->active)
+		else if(!strnicmp(reg->type, "ext", 3) && !reg->traffic->active)
 			data[idx].status = "offline";
+		else if(!strnicmp(reg->type, "ext", 3) && reg->traffic->getTimer() > 0)
+		{
+			switch(reg->traffic->presence)
+			{
+			case SIPTraffic::P_AWAY:
+				data[idx].status = "away";
+				break;
+			case SIPTraffic::P_BUSY:
+				data[idx].status = "busy";
+				break;
+			case SIPTraffic::P_DND:
+				data[idx].status = "dnd";
+				break;
+			default:
+				if(!reg->call_limit || reg->traffic->active_calls < reg->call_limit)
+					data[idx].status = "ready";
+				else
+					data[idx].status = "busy";
+			}
+		}
 		else if(!reg->proxy || !reg->userid)
 			data[idx].status = "invalid";
-		else if(!reg->active)
+		else if(!reg->traffic->active)
 			data[idx].status = "failed";
 		else if(reg->getTimer() > 0)
-			data[idx].status = "active";
+		{
+			if(!reg->call_limit || reg->traffic->active_calls < reg->call_limit)
+				data[idx].status = "active";
+			else
+				data[idx].status = "full";
+		}
 		else
 			data[idx].status = "expired";
-		data[idx].updated = reg->updated;
-		data[idx].attempts_iCount = reg->call_attempts.iCount;
-		data[idx].attempts_oCount = reg->call_attempts.oCount;
-		data[idx].complete_iCount = reg->call_complete.iCount;
-		data[idx].complete_oCount = reg->call_complete.oCount;
-		data[idx].active_calls = reg->active_calls;
+		data[idx].updated = reg->traffic->updated;
+		data[idx].attempts_iCount = reg->traffic->call_attempts.iCount;
+		data[idx].attempts_oCount = reg->traffic->call_attempts.oCount;
+		data[idx].complete_iCount = reg->traffic->call_complete.iCount;
+		data[idx].complete_oCount = reg->traffic->call_complete.oCount;
+		data[idx].active_calls = reg->traffic->active_calls;
 		data[idx].call_limit = reg->call_limit;
 		++idx;
 		reg = reg->next;
@@ -779,7 +856,6 @@ const char *Driver::assignScript(ScriptImage *img, Line *line)
 		memset(reg, 0, sizeof(Registry));
 		reg->add(img);
 		reg->proxy = NULL;
-		reg->active = true;
 		reg->protocol = "sip";
 		reg->iface = "none";
 		reg->uri = cp;
@@ -792,7 +868,6 @@ const char *Driver::assignScript(ScriptImage *img, Line *line)
 		reg->secret = NULL;
 		reg->line = line;
 		reg->duration = 0;		
-		reg->updated = 0;
 
 		setString(cbuf, sizeof(cbuf), scr->name);
 		ep = strchr(cbuf, ':');
@@ -852,6 +927,8 @@ const char *Driver::assignScript(ScriptImage *img, Line *line)
                 reg->localid = (char *)img->getMemory(strlen(uname) + 1);
                 strcpy((char *)reg->localid, uname);
 		slog.debug("sip: assigning %s to %s", scr->name, buffer);
+		reg->traffic = Driver::sip.getTraffic(rname);
+		reg->traffic->active = true;
 		img->setPointer(rname, reg);
 		if(!cp)
 			break;
@@ -933,6 +1010,75 @@ void Driver::updateExpires(Registry *reg, timeout_t expires)
 	eXosip_unlock();
 }
 
+void Driver::publishRequest(eXosip_event_t *sevent, const char *target)
+{
+	osip_authorization_t *auth = NULL;
+	char buffer[65];
+	Registry *reg;
+	ScriptImage *img;
+
+	if(!osip_message_get_authorization(sevent->request, 0, &auth))
+		goto registerAuth;
+
+	if(!target)
+		goto registerAuth;
+
+	snprintf(buffer, sizeof(buffer), "uri.%s", target);
+	server->enter();
+	img = server->getActive();
+	reg = (Registry *)img->getPointer(buffer);	
+	if(reg && reg->insecure_publish)
+	{
+		publishUpdate(sevent, reg);
+		server->leave();
+		return;
+	}
+
+	server->leave();
+registerAuth:
+	registerRequest(sevent);
+}
+
+void Driver::publishUpdate(eXosip_event_t *sevent, Registry *reg)
+{
+	const char *tmp;
+	bool online = true;
+	osip_body_t *mbody = NULL;
+	osip_message_get_body(sevent->request, 0, &mbody);
+	bool presence = false;
+
+	if(!mbody)
+		return;
+
+	tmp = mbody->body;
+
+	while(*tmp)
+	{
+		if(!strnicmp(tmp, "<presence", 9))
+			presence = true; 
+
+		if(!strnicmp(tmp, "online", 6) && presence)
+			break;
+
+		else if(!strnicmp(tmp, "busy", 4) && presence)
+		{
+			reg->traffic->presence = SIPTraffic::P_BUSY;
+			online = false;
+			break;
+		}
+		else if(!strnicmp(tmp, "away", 4) && presence)
+		{
+			reg->traffic->presence = SIPTraffic::P_AWAY;
+			online = false;
+			break;
+		}
+		++tmp;
+	}
+
+	if(online && presence)
+		reg->traffic->presence = SIPTraffic::P_ONLINE;
+}
+	
 void Driver::registerRequest(eXosip_event_t *sevent)
 {
 	char a1[256], a2[256], a1_hash[256], a2_hash[256], resp[256], resp_hash[256];
@@ -947,6 +1093,7 @@ void Driver::registerRequest(eXosip_event_t *sevent)
 	Registry *reg;	
 	int pos = 0;
 	char *cp;
+	InetAddress addr;
 
 	eXosip_lock();
 	if(osip_message_get_authorization(sevent->request, 0, &auth) != 0)
@@ -1001,18 +1148,14 @@ fail401:
 	if(!stricmp(reg->type, "peer"))
 		goto sip200;
 
-	snprintf(buf, sizeof(buf), 
-		"%s:%s", contact->url->host, contact->url->port);
-
-
-	if(reg->proxy)
-		printf("PROXY %s TO %s\n", reg->proxy, buf);
+	addr = contact->url->host;
+	snprintf(buf, sizeof(buf), "%s:%s", 
+		inet_ntoa(addr.getAddress()), contact->url->port);
 
 	if(!reg->proxy || stricmp(buf, reg->proxy))
 	{
-		cp = (char *)img->getMemory(strlen(buf));
-		strcpy(cp, buf);
-		reg->proxy = cp;
+		setString(reg->traffic->address, sizeof(reg->traffic->address), buf);
+		reg->proxy = reg->traffic->address;
 	}
 
 	if(!reg->userid || stricmp(contact->url->username, reg->userid))
@@ -1023,6 +1166,11 @@ fail401:
 	}
 
 sip200:
+	if(MSG_IS_PUBLISH(sevent->request))
+	{
+		publishUpdate(sevent, reg);
+		goto answer200;
+	}
 	eXosip_lock();	
 	osip_message_get_expires(sevent->request, 0, &exp);
 	if(exp && exp->hvalue)
@@ -1035,18 +1183,23 @@ sip200:
 	eXosip_unlock();
 	if(pos)
 	{
-		time(&reg->updated);
-		reg->setTimer(pos * 1000);
-		reg->active = true;
+		time(&reg->traffic->updated);
+		if(!reg->traffic->getTimer())
+			reg->traffic->presence = SIPTraffic::P_ONLINE;
+
+		reg->traffic->setTimer(pos * 1000);
+		reg->traffic->active = true;
 		slog.debug("registered external %s from %s; expires=%d", auth->username, buf, pos);
 	}
 	else
 	{
-		reg->updated = 0l;
-		reg->active = false;
-		reg->setTimer(0);
+		reg->traffic->updated = 0l;
+		reg->traffic->active = false;
+		reg->traffic->setTimer(0);
+		reg->traffic->presence = SIPTraffic::P_AVAILABLE;
 		slog.debug("unregister external %s from %s", auth->username, buf);
 	}
+answer200:
 	server->leave();
 
         eXosip_lock();
@@ -1068,19 +1221,20 @@ const char *Driver::registerScript(ScriptImage *img, Line *line)
 {
 	Name *scr = img->getCurrent();
 	Registry *reg = (Registry *)img->getMemory(sizeof(Registry));
-        const char *cp = ScriptChecks::findKeyword(line, "timeout");
-	const char *lcp;
+        const char *dur = ScriptChecks::findKeyword(line, "timeout");
+	const char *insecure = ScriptChecks::findKeyword(line, "insecure");
+	const char *lcp, *cp;
 	char buf[1024];
 	char nbuf[32];
 	unsigned len, pos;
 	char *p;
 	char *uri;
-	long mult = 60000;
+	long mult = 1000;
 	osip_message_t *msg = NULL; 
 	InetAddress host;
 	bool ext = false;
-	Registry *preg;
-	ScriptImage *pimg;
+	Image *ip = (Image *)img;
+	Keydata *cfg = NULL;
 
 	memset(reg, 0, sizeof(Registry));
 
@@ -1090,7 +1244,7 @@ const char *Driver::registerScript(ScriptImage *img, Line *line)
 	if(ext)
 	{
 		if(!ScriptChecks::useKeywords(line, 
-			"=secret=encoding=framing=dtmf=forward=user"))
+			"=secret=encoding=framing=dtmf=user=count=device"))
 				return "invalid keywords used for sip external";
 	}
 	else if(!ScriptChecks::useKeywords(line, 
@@ -1098,8 +1252,8 @@ const char *Driver::registerScript(ScriptImage *img, Line *line)
 		"=limit=public=encoding=framing=local=caller"))
 			return "invalid keywords for sip registry";
 
-	reg->updated = 0l;
-	reg->active = false;
+
+
 	reg->protocol = "sip";
 	reg->iface = getLast("interface");
 	reg->uri = ScriptChecks::findKeyword(line, "uri");
@@ -1111,7 +1265,6 @@ const char *Driver::registerScript(ScriptImage *img, Line *line)
 	reg->type = ScriptChecks::findKeyword(line, "type");
 	reg->dtmf = ScriptChecks::findKeyword(line, "dtmf");
 	reg->encoding = ScriptChecks::findKeyword(line, "encoding");
-	reg->forward = ScriptChecks::findKeyword(line, "forward");
 	reg->caller = ScriptChecks::findKeyword(line, "caller");
 
 	if(ext)
@@ -1120,6 +1273,9 @@ const char *Driver::registerScript(ScriptImage *img, Line *line)
 		reg->localid = ScriptChecks::findKeyword(line, "local");
 
 	cp = ScriptChecks::findKeyword(line, "limit");
+	if(cp)
+		reg->call_limit = atoi(cp);
+	cp = ScriptChecks::findKeyword(line, "count");
 	if(cp)
 		reg->call_limit = atoi(cp);
 
@@ -1127,6 +1283,66 @@ const char *Driver::registerScript(ScriptImage *img, Line *line)
 	if(!cp || !reg->encoding)
 		cp = "0";
 	reg->framing = atol(cp);
+	
+#ifndef	WIN32
+	cp = ScriptChecks::findKeyword(line, "device");
+	if(cp)
+	{
+		snprintf(buf, sizeof(buf), "/bayonne/sipdevice/%s", cp);
+		cfg = new Keydata(buf);
+	}
+
+	if(cfg)
+	{
+		cp = cfg->getLast("realm");
+		if(cp && !reg->realm)
+			reg->realm = ip->dupString(cp);
+		cp = cfg->getLast("type");
+		if(cp && !reg->type)
+			reg->type = cp;
+		cp = cfg->getLast("count");
+		if(cp && !reg->call_limit)
+			reg->call_limit = atoi(cp);
+			
+		cp = cfg->getLast("dtmf");
+		if(cp && !reg->dtmf)
+			reg->dtmf = ip->dupString(cp);					
+
+		cp = cfg->getLast("secret");
+		if(cp && !reg->secret)
+			reg->secret = ip->dupString(cp);
+
+		if(!insecure)
+			insecure = cfg->getLast("insecure");
+
+		cp = cfg->getLast("user");
+		if(cp && !reg->userid)
+			reg->userid = ip->dupString(cp);
+
+		cp = cfg->getLast("encodings");
+		if(cp && !strstr(cp, getLast("encoding")))
+			cp = cfg->getLast("preferred");
+		else
+			cp = NULL;
+		if(!cp)
+			cp = cfg->getLast("encoding");
+		if(cp && !reg->encoding)
+		{
+			reg->encoding = ip->dupString(cp);
+			cp = cfg->getLast("framing");
+			if(cp)
+				reg->framing = atol(cp);
+		}
+		delete cfg;
+	}
+#endif
+
+	while(insecure && *insecure)
+	{
+		if(!strnicmp(insecure, "publish", 7))
+			reg->insecure_publish = true;
+		++insecure;
+	} 
 
 	reg->add(img);
 
@@ -1178,8 +1394,9 @@ const char *Driver::registerScript(ScriptImage *img, Line *line)
 	if(!*reg->secret)
 		reg->secret = NULL;	
 
-	if(!cp)
-		cp = getLast("duration");
+	if(!dur)
+		dur = getLast("duration");
+	cp = dur;
 
 	lcp = cp + strlen(cp) - 1;
 	switch(*lcp)
@@ -1283,29 +1500,6 @@ const char *Driver::registerScript(ScriptImage *img, Line *line)
 	if(ext)
 		reg->type = "extern";
 
-	pimg = useImage();
-	preg = NULL;
-	if(pimg)
-	{
-		snprintf(buf, sizeof(buf), "uri.%s", reg->localid);
-		preg = (Registry *)pimg->getPointer(buf);
-	}
-			
-	if(preg && preg->active && !stricmp(reg->type, preg->type))
-	{
-		reg->active = true;
-		reg->setTimer(preg->getTimer());
-		if(ext && preg->proxy)
-		{
-			reg->proxy = (const char *)img->getMemory(strlen(preg->proxy) + 1);
-			strcpy((char *)reg->proxy, preg->proxy);
-			reg->updated = preg->updated;
-		}
-	}
-
-	if(pimg)
-		endImage(pimg);
-
 	if(ext)
 	{
 		slog.debug("register %s externally", reg->localid);
@@ -1376,6 +1570,9 @@ const char *Driver::registerScript(ScriptImage *img, Line *line)
 
 add:
 	snprintf(buf, sizeof(buf), "uri.%s", reg->localid);
+	reg->traffic = Driver::sip.getTraffic(buf);
+	if(reg->traffic->address[0])
+		reg->proxy = reg->traffic->address;
 	img->setPointer(buf, reg);
 	img->addRegistration(line);
 	line->scr.registry = reg;
@@ -1481,13 +1678,13 @@ void Driver::run(void)
 				expires = reg->duration;
 			if(reg && !expires)
 			{
-				reg->active = false;
+				reg->traffic->active = false;
 				slog.debug("registration for %s rejected", reg->contact);
 			}
 			else if(reg)
 			{
-				reg->active = true;
-				reg->setTimer(expires);
+				reg->traffic->active = true;
+				reg->traffic->setTimer(expires);
 				if(expires != reg->duration)
 				{
 					updateExpires(reg, expires);
@@ -1515,8 +1712,8 @@ void Driver::run(void)
 			}
 			if(reg)
 			{
-				reg->setTimer(0);
-				reg->active = false;
+				reg->traffic->setTimer(0);
+				reg->traffic->active = false;
 				slog.info("registration for %s failed", reg->contact);
 			}
 			else
@@ -1837,10 +2034,62 @@ killit:
 				session->postEvent(&event);
 			break;
 		case EXOSIP_MESSAGE_NEW:
+                        local_media = NULL;
+                        conn = NULL;
+                        remote_sdp = NULL;
+                        local_sdp = NULL;
+                        display = NULL;
+                        caller = NULL;
+                        dialed = NULL;
+                        local_uri = NULL;
+                        remote_uri = NULL;
+                        from = NULL;
+                        to = NULL;
+                        sdp = NULL;
+                        msg = NULL;
+                        session = NULL;
+
+
 			if(!sevent->request)
 				break;
 			if(MSG_IS_REGISTER(sevent->request))
 				registerRequest(sevent);
+			else if(MSG_IS_PUBLISH(sevent->request))
+			{
+				osip_to_to_str(sevent->request->to, &local_uri);
+				if(!local_uri)
+					goto cleanup;
+				osip_from_to_str(sevent->request->from, &remote_uri);
+				if(!remote_uri)
+					goto cleanup;
+				osip_to_init(&to);
+				osip_to_parse(to, local_uri);
+				if(!to)
+					goto cleanup;
+
+				osip_from_init(&from);
+				osip_from_parse(from, remote_uri);
+				if(!from)
+					goto cleanup;
+
+				if(!from->url || !to->url)
+					goto cleanup;
+
+				if(!from->url->username || !to->url->username)
+					goto cleanup;
+
+				if(stricmp(from->url->username, to->url->username))
+					goto cleanup;
+
+				if(stricmp(to->url->port, from->url->port))
+					goto cleanup;
+
+				if(stricmp(to->url->host, from->url->host))
+					goto cleanup;
+
+				publishRequest(sevent, to->url->username);
+				goto cleanup;
+			}
 			break;
 		case EXOSIP_CALL_INVITE:
 			if(!sevent->request)
@@ -2001,7 +2250,7 @@ validate:
 				if(!reg->isActive())
 					goto sip404;
 	
-				if(reg->call_limit && reg->active_calls >= reg->call_limit)
+				if(reg->call_limit && reg->traffic->active_calls >= reg->call_limit)
 					goto sip480;
 
 				scr = img->getScript(reg->scr->name);
@@ -2092,9 +2341,9 @@ sip480:
 
 			if(reg)
 			{
-				++reg->active_calls;
-				++reg->call_attempts.iCount;
-				++reg->call_complete.iCount;
+				++reg->traffic->active_calls;
+				++reg->traffic->call_attempts.iCount;
+				++reg->traffic->call_complete.iCount;
 				if(!stricmp(reg->type, "anon") || !strnicmp(reg->type, "ext", 3))
 					snprintf(cbuf, sizeof(cbuf), "uri.%s", reg->localid);
 				else
@@ -2356,6 +2605,7 @@ clear:
 			if(session)
 				add(session);
 
+cleanup:
 			eXosip_lock();
 //			if(msg)
 //				osip_message_free(msg);
