@@ -71,10 +71,47 @@ struct dtmf2833
 
 class Session;
 
+class Image : public ScriptImage
+{
+private:
+	Image() : ScriptImage(NULL, NULL) {};
+
+public:
+	const char *dupString(const char *str);
+};
+
+class SIPTraffic : public TimerPort
+{
+private:
+	friend class Driver;
+
+	static SIPTraffic *first;
+	SIPTraffic *next;
+
+public:
+	Bayonne::Traffic call_attempts, call_complete;
+	unsigned refcount;
+	volatile short active_calls;
+	time_t updated;
+	bool active;
+	enum
+	{
+		P_AVAILABLE = 0,
+		P_BUSY,
+		P_AWAY,
+		P_DND,
+
+		P_ONLINE = P_AVAILABLE
+	} presence;
+	
+	char address[48];
+};	
+
 class Registry : public ScriptRegistry
 {
 public:
 	Registry *next;
+	SIPTraffic *traffic;
 	const char *uri, *contact, *proxy;
 	const char *iface;
 //	const char *route;
@@ -87,16 +124,14 @@ public:
 	const char *realm;
 	const char *dtmf;
 	const char *encoding;
-	const char *forward;
 	const char *caller;
 	timeout_t framing;
 	int regid;
-	bool active;
-	time_t updated;
-	Bayonne::Traffic call_attempts, call_complete;
-	volatile short active_calls;
 	short call_limit;
 
+	bool insecure_publish;
+
+
 	bool isActive(void);
 	void add(ScriptImage *img);
 };
@@ -114,7 +149,7 @@ private:
 	SIPThread();
 };
 
-class Driver : public BayonneDriver, public Audio, public Thread
+class Driver : public BayonneDriver, public Audio, public Thread, public Assoc
 {
 protected:
 	friend class Session;
@@ -138,6 +173,8 @@ protected:
 	starting_mode_t starting;
 	static unsigned instance;
 
+	void *getMemory(size_t size);
+
 public:
 	static Driver sip;		// plugin activation
 
@@ -148,9 +185,14 @@ public:
 
 	void startDriver(void);
 	void stopDriver(void);
+	void reloadDriver(void);
+
+	SIPTraffic *getTraffic(const char *id);
 
 	void requestAuth(eXosip_event_t *sevent);
 	bool getRegistry(eXosip_event_t *sevent, Registry **reg, ScriptImage *img);
+	void publishRequest(eXosip_event_t *sevent, const char *target);
+	void publishUpdate(eXosip_event_t *sevent, Registry *reg);
 	void registerRequest(eXosip_event_t *sevent);
 	const char *registerScript(ScriptImage *img, Line *line);
 	const char *assignScript(ScriptImage *img, Line *line);
