@@ -276,6 +276,7 @@ bool Session::enterSeize(Event *event)
 	const char *caller = NULL;
 	Name *scr;
 	unsigned idx;
+	bool offline = false;
 
 	switch(event->id)
 	{
@@ -313,9 +314,17 @@ bool Session::enterSeize(Event *event)
 				if(strnicmp(reg->type, "ext", 3))
 					goto single;
 				if(!reg->isActive())
+				{
+					offline = true;
 					continue;
+				}
 				goto dialer;
 			}
+			if(offline)
+			{
+				event->id = DIAL_OFFLINE;
+				return false;
+			}
 		}
 single:
 		cp = getSymbol("session.dialed");
@@ -324,7 +333,7 @@ single:
 			event->id = DIAL_FAILED;
 			return false;
 		}
-forward:
+//forward:
 		if(!strchr(cp, '@') && !route)
 		{
 			if(!strnicmp(cp, "sip:", 4))
@@ -336,20 +345,30 @@ forward:
 			{
 				if(!reg->isActive())
 				{
-					cp = reg->forward;
-					if(cp)
-						goto forward;
-
-					event->id = DIAL_FAILED;
+					event->id = DIAL_OFFLINE;
 					return false;
 				}
 				if(!stricmp(reg->type, "anon"))
 				{
-					event->id = DIAL_FAILED;
+					event->id = DIAL_INVALID;
 					return false;
 				}
 dialer:
-				if(reg->call_limit && reg->active_calls >= reg->call_limit)
+				switch(reg->traffic->presence)
+				{
+				case SIPTraffic::P_BUSY:
+					event->id = DIAL_BUSY;
+					return false;
+				case SIPTraffic::P_AWAY:
+					event->id = DIAL_AWAY;
+					return false;
+				case SIPTraffic::P_DND:
+					event->id = DIAL_DND;
+					return false;
+				default:
+					break;
+				}
+				if(reg->call_limit && reg->traffic->active_calls >= reg->call_limit)
 				{
 					event->id = DIAL_BUSY;
 					return false;
@@ -402,9 +421,9 @@ dialing:
 			if(!route)
 				route = reg->proxy;
 
-			++reg->active_calls;
-			++reg->call_attempts.oCount;
-			++reg->call_complete.oCount;
+			++reg->traffic->active_calls;
+			++reg->traffic->call_attempts.oCount;
+			++reg->traffic->call_complete.oCount;
 			if(!strnicmp(reg->type, "ext", 3))
 				snprintf(rbuf, sizeof(rbuf), 
 					"uri.%s", reg->localid);
@@ -607,7 +626,7 @@ bool Session::enterHangup(Event *event)
 			break;
 		reg = (Registry *)img->getPointer(cp);
 		if(reg)
-			--reg->active_calls;	
+			--reg->traffic->active_calls;	
 		break;
 	case TIMER_EXPIRED:
 		if(!rtp_flag)
