@@ -59,26 +59,46 @@ void JoinThread::peerAudio(Encoded encoded)
 void JoinThread::run(void)
 {
 	BayonneSession *peer;
+	BayonneSession *parent = Bayonne::getSid(session->var_pid);
 	const char *errmsg = NULL;
 	bool exiting;
 
-	switch(Bayonne::peer_encoding)
+	Audio::Encoding encoding = 
+		Audio::getEncoding(Driver::voicetronix.getLast("encoding"));
+	timeout_t framing = 
+		atol(Driver::voicetronix.getLast("framing"));
+
+	if(parent)
+	{
+		encoding = getEncoding(parent->audioEncoding());
+		framing = parent->audioFraming();
+	}
+			
+	switch(encoding)
 	{
 	case mulawAudio:
 		vpb_play_buf_start(handle, VPB_MULAW);
 		vpb_record_buf_start(handle, VPB_MULAW);
-		bufcount = bufsize = Bayonne::peer_framing * 8;
+		bufcount = bufsize = framing * 8;
 		break;
 	case alawAudio:
                 vpb_play_buf_start(handle, VPB_ALAW);
                 vpb_record_buf_start(handle, VPB_ALAW); 
-                bufcount = bufsize = Bayonne::peer_framing * 8;
+                bufcount = bufsize = framing * 8;
                 break; 
 	case pcm16Mono:
                 vpb_play_buf_start(handle, VPB_LINEAR); 
                 vpb_record_buf_start(handle, VPB_LINEAR);  
-                bufsize = Bayonne::peer_framing * 16;
-		bufcount = Bayonne::peer_framing * 8; 
+                bufsize = framing * 16;
+		bufcount = framing * 8; 
+		break;
+	case okiADPCM:
+		vpb_play_buf_start(handle, VPB_OKIADPCM);
+		vpb_record_buf_start(handle, VPB_OKIADPCM);
+		bufsize = framing * 4;
+		bufcount = framing * 8;
+	default:
+		break;
 	}
 
 	reset = false;
@@ -237,11 +257,6 @@ void PlayThread::run(void)
 		bufsize = 80;
 		bufcount = 160;
 		playDirect();
-	case g723_3bit:
-		vpb_play_buf_start(handle, VPB_OKIADPCM24);
-		bufsize = 60;
-		bufcount = 160;
-		playDirect();
 	default:
 		playConvert();
 	}
@@ -388,11 +403,6 @@ void RecordThread::run(void)
                 bufsize = 80;
                 bufcount = 160;
                 recordDirect();
-        case g723_3bit:
-                vpb_record_buf_start(handle, VPB_OKIADPCM24);
-                bufsize = 60;
-                bufcount = 160;
-                recordDirect();
 	default:
 convert:
 		recordConvert();
@@ -508,7 +518,7 @@ void RecordThread::recordDirect(void)
 			errmsg = "record-failed";
 			break;
 		}
-		if(audio->putBuffer(buffer, bufsize) < bufsize)
+		if(audio->putBuffer(buffer, bufsize) < (int)bufsize)
 		{
 			errmsg = "write-failed";
 			break;
